#ifndef OBJECT_H
#define OBJECT_H

#include "asciibmp.h"

const int move_pattern[8][2] = //[facing][x/y]
{
	{0, -1},
	{1, -1},
	{1, 0},
	{1, 1},
	{0, 1},
	{-1, 1},
	{-1, 0},
	{-1, -1}
};

/*Each instance represents an object on the map, such as trees etc. Railways are no objects. For each object type, a blueprint
is created at startup from an objectfile, each real instance is copied from such a blueprint. */
class object
{
protected:
	bool destroy_graphics; //when instance is deleted, delete graphics (only true for blueprints, not actual instances which get a pointer only)
    
	int pos[2]; //in characters
	asciibmp* pic; //picture representing object

/* DONE (Deathlord#1#): Replace Bitmap with asciibmp */

	virtual void fill(); //set proper values    

public:
	object() {fill();} //empty instance
	object(std::string objectfile); //make a blueprint (used at startup)
	object(const object &from_blueprint, int posx, int posy); //create a "real" instance from a blueprint instance
	virtual ~object();
	
	virtual void write_objectfile(std::string to, std::string picpath); //picpath: no file extension
	virtual void load_objectfile(std::string from);
	
	bool collides(const object *with); //returns true if the two objects collide;
	
    //setters / getters
	inline bool get_destroy_graphics() const {return destroy_graphics;}
	inline int get_posx() const {return pos[0];}
	inline int get_posy() const {return pos[1];}
	virtual asciibmp* get_pic() const {return pic;}
    
	inline void set_destroy_graphics(bool v) {destroy_graphics = v;}
	inline void set_pos(int x, int y) {pos[0] = x; pos[1] = y;}
	inline void set_pic(asciibmp* to) {pic = to;}
};

#endif
